#include <node_api.h>
#include <stdlib.h>

//-----------------------------------------------------------------------------
// generic_throw_error()
//-----------------------------------------------------------------------------
void generic_throw_error(napi_env env)
{
    const napi_extended_error_info *errorInfo;
    const char *errorMessage;
    bool isPending;

    napi_get_last_error_info(env, &errorInfo);
    napi_is_exception_pending(env, &isPending);
    if (!isPending) {
        errorMessage = errorInfo->error_message;
        if (!errorMessage)
            errorMessage = "no error message?";
        napi_throw_error(env, NULL, errorMessage);
    }
}

//-----------------------------------------------------------------------------
// issue_26667_finalize()
//-----------------------------------------------------------------------------
static void issue_26667_finalize(napi_env env, void *finalizeData,
        void *finalizeHint)
{
    free(finalizeData);
}


//-----------------------------------------------------------------------------
// issue_26667_test()
//-----------------------------------------------------------------------------
static napi_value issue_26667_test(napi_env env, napi_callback_info info)
{
    napi_value instance;
    void *data;

    // allocate an arbitrary amount of memory
    data = malloc(32);
    if (!data)
        napi_fatal_error(NULL, 0, "out of memory!", NAPI_AUTO_LENGTH);

    // create new instance
    if (napi_create_external_buffer(env, 32, data, issue_26667_finalize, data, &instance) != napi_ok) {
        generic_throw_error(env);
        return NULL;
    }

    return instance;
}


//-----------------------------------------------------------------------------
// issue_26667_init()
//-----------------------------------------------------------------------------
static napi_value issue_26667_init(napi_env env, napi_value exports)
{
    napi_value fn;

    // define function for performing the test
    if (napi_create_function(env, "test", NAPI_AUTO_LENGTH,
            issue_26667_test, NULL, &fn) != napi_ok) {
        generic_throw_error(env);
        return NULL;
    }
    if (napi_set_named_property(env, exports, "test", fn) != napi_ok) {
        generic_throw_error(env);
        return NULL;
    }

    return exports;
}

NAPI_MODULE(NODE_GYP_MODULE_NAME, issue_26667_init)
